########################################################################
# PACKAGE: Minas::LDOCE
#
# Copyright (C), 2002
# Bridget McInnes,       bthomson@d.umn.edu
# Deodatta Y Bhoite,     bhoi0001@d.umn.edu
# Kailash Aurangabadkar, aura0011@d.umn.edu
# Nitin Agarwal,         agar0054@d.umn.edu
# Yanhua Li,             lixx0333@d.umn.edu
#
# University of Minnesota, Duluth
#
# USAGE:
#           use Minas::LDOCE
#
# The package LDOCE allows a user to access different aspects of 
# the LDOCE3 machine readable dictionary. For more information 
# please view the perl doc
#
#           perldoc Minas::LDOCE
#
#######################################################################
package Minas::LDOCE;

use 5.008;
use strict;

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Minas::LDOCE ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw() ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw();

our $VERSION = '1.0';

###############
# new method  #
###############
my $location;
sub new
{
    # First argument is class
    my $class = shift;

    # Second is location of LDOCE3 SGL files; Third is verbosity
    $location =  shift;

    my $self = {};

    bless $self, $class;

    $self->{dir} = shift if (defined(@_ > 0));
    $self->{verbose} = @_ ? shift : 0;

    warn "Dir = ", $self->{dir}, "\n" if ($self->{verbose});
    warn "Verbose = ", $self->{verbose}, "\n" if ($self->{verbose});

    if(-f "index") {} else { createindexfile(); }
    if(-f "pos") {} else { createposfile(); }
    if(-f "active") {} else { createactivefile(); }

    return $self;
}

##########################################
#  Create subject, active and pos files  #
##########################################
sub createactivefile
{
    my $word = "";
    my @sgmfiles = ( 'A.SGM','B.SGM','C.SGM','D.SGM','e.sgm','F.SGM','G.SGM',
		     'H.SGM','I.SGM','J.SGM','K.SGM','L.SGM','M.SGM','N.SGM',
		     'O.SGM','P.SGM','Q.SGM','R.SGM','S.SGM','T.SGM','U.SGM',
		     'V.SGM','W.SGM','X.SGM','Y.SGM','Z.SGM');
    # open the index file
    open(ACTIVE, ">active");
    foreach my $sgmfile (@sgmfiles) {
	$sgmfile = $location . $sgmfile;
	open(SGM, $sgmfile) or die (print "$sgmfile \n");
	while(<SGM>) {
	    if($_=~m/<Entry><Head><HWD>([A-Za-z]+)<\/HWD>/) { $word = $1; }
	    if($_=~ /<ACTIV>([A-Z]+)<\/ACTIV>/) { print ACTIVE "$word $1\n";}
	}	
    }
    close(ACTIVE);
}

sub createindexfile
{
    my $word = "";
    my @sgmfiles = ( 'A.SGM','B.SGM','C.SGM','D.SGM','e.sgm','F.SGM','G.SGM',
		     'H.SGM','I.SGM','J.SGM','K.SGM','L.SGM','M.SGM','N.SGM',
		     'O.SGM','P.SGM','Q.SGM','R.SGM','S.SGM','T.SGM','U.SGM',
		     'V.SGM','W.SGM','X.SGM','Y.SGM','Z.SGM');
    # open the index file
    open(INDEX, ">index");
    foreach my $sgmfile (@sgmfiles) {
	$sgmfile = $location . $sgmfile;
	open(SGM, $sgmfile) or die(print "$sgmfile \n");
	while(<SGM>) {
	    if($_=~m/<Entry><Head><HWD>([A-Za-z]+)<\/HWD>/) { $word = $1; }
	    if($_=~ /<FIELD>([A-Z]+)<\/FIELD>/) { print INDEX "$word $1\n";}
	}	
    }
    close(INDEX);
}

sub createposfile
{
    my $word = "";
    my @sgmfiles = ( 'A.SGM','B.SGM','C.SGM','D.SGM','e.sgm','F.SGM','G.SGM',
		     'H.SGM','I.SGM','J.SGM','K.SGM','L.SGM','M.SGM','N.SGM',
		     'O.SGM','P.SGM','Q.SGM','R.SGM','S.SGM','T.SGM','U.SGM',
		     'V.SGM','W.SGM','X.SGM','Y.SGM','Z.SGM');
    # open the index file
    open(POS, ">pos");
    foreach my $sgmfile (@sgmfiles) {
	$sgmfile = $location . $sgmfile;
	open(SGM, $sgmfile) or die(print "$sgmfile \n");
	while(<SGM>) {
	    if($_=~m/<Entry><Head><HWD>(.+)<\/HWD>/) { $word = $1; }
	    if($_=~ /<POS>(.+)<\/POS>/) { print POS "$word:$1\n"; }
	}	
    }
    close(POS);
}


#############################
# Private Package Variables #
#############################
my %parent_field_codes =  ('A'     => 'ARTS', 
			   'AA'    => 'ARTS', 
			   'AL'    => 'ARTS', 
			   'AM'    => 'ARTS', 
			   'AMF'   => 'Media', 
			   'AMT'   => 'Media', 
			   'AP'    => 'ARTS', 
			   'APD'   => 'Performing', 
			   'APM'   => 'Performing', 
			   'APT'   => 'Performing', 
			   'AV'    => 'ARTS', 
			   'AVD'   => 'Visual', 
			   'AVP'   => 'Visual', 
			   'AVS'   => 'Visual', 
			   'B'     => 'BUSINESS', 
			   'BB'    => 'BUSINESS', 
			   'BBA'   => 'Basics', 
			   'BBC'   => 'Basics', 
			   'BBB'   => 'Basics', 
			   'REM'   => 'Basics', 
			   'BBO'   => 'Basics', 
			   'RET'   => 'Basics', 
			   'BE'    => 'BUSINESS', 
			   'BEC'   => 'Employment', 
			   'BEL'   => 'Employment', 
			   'BEW'   => 'Employment', 
			   'BF'    => 'BUSINESS', 
			   'BFB'   => 'Finance', 
			   'BFL'   => 'Finance', 
			   'BFS'   => 'Finance', 
			   'BFI'   => 'Finance', 
			   'BO'    => 'BUSINESS', 
			   'C'     => 'COLOURS/SOUNDS/ETC', 
			   'CC'    => 'COLOURS/SOUNDS/ETC', 
			   'CF'    => 'COLOURS/SOUNDS/ETC', 
			   'CO'    => 'COLOURS/SOUNDS/ETC', 
			   'CS'    => 'COLOURS/SOUNDS/ETC', 
			   'CT'    => 'COLOURS/SOUNDS/ETC', 
			   'D'     => 'DAILYLIFE', 
			   'DC'    => 'Clothes&Fashion', 
			   'DCB'   => 'Clothes&Fashion', 
			   'DCC'   => 'Clothes&Fashion', 
			   'DCJ'   => 'Clothes&Fashion', 
			   'DF'    => 'DAILYLIFE', 
			   'DFC'   => 'Food', 
			   'DFD'   => 'Food', 
			   'DFF'   => 'Food', 
			   'DFN'   => 'Food', 
			   'DFT'   => 'Food', 
			   'DFU'   => 'Food', 
			   'DG'    => 'DAILYLIFE', 
			   'DOB'   => 'Games', 
			   'DGC'   => 'Games', 
			   'DOD'   => 'Games', 
			   'DOG'   => 'Games', 
			   'DGO'   => 'Games', 
			   'DOS'   => 'Games', 
			   'DH'    => 'DAILYLIFE', 
			   'DHB'   => 'Household', 
			   'DHC'   => 'Household', 
			   'DHF'   => 'Household', 
			   'DHH'   => 'Household', 
			   'DHP'   => 'Household', 
			   'DHT'   => 'Household', 
			   'DL'    => 'DAILYLIFE', 
			   'DLA'   => 'Leisure', 
			   'DLC'   => 'Leisure', 
			   'DLG'   => 'Leisure', 
			   'DLH'   => 'Leisure', 
			   'DLO'   => 'Leisure', 
			   'DLT'   => 'Leisure', 
			   'DN'    => 'DAILYLIFE', 
			   'DS'    => 'DAILYLIFE', 
			   'DSA'   => 'Sports', 
			   'DSB'   => 'Sports', 
			   'DSC'   => 'Sports', 
			   'DSF'   => 'Sports', 
			   'DSG'   => 'Sports', 
			   'DSH'   => 'Sports', 
			   'DSO'   => 'Sports', 
			   'DSS'   => 'Sports', 
			   'DST'   => 'Sports', 
			   'H'     => 'HARDSCIENCE', 
			   'HA'    => 'HARDSCIENCE', 
			   'HB'    => 'HARDSCIENCE', 
			   'HBA'   => 'Biology', 
			   'HBB'   => 'Biology', 
			   'HBF'   => 'Biology', 
			   'HBH'   => 'Biology', 
			   'HBI'   => 'Biology', 
			   'HBM'   => 'HARDSCIENCE', 
			   'HBP'   => 'Microbes/Genetics/Biochem', 
			   'HC'    => 'HARDSCIENCE', 
			   'HCC'   => 'Chemistry', 
			   'HCE'   => 'Chemistry', 
			   'HCM'   => 'Chemistry', 
			   'HCP'   => 'Chemistry', 
			   'HE'    => 'HARDSCIENCE', 
			   'MEG'   => 'EarthSciences', 
			   'HEG'   => 'EarthSciences', 
			   'HEO'   => 'EarthSciences', 
			   'HM'    => 'HARDSCIENCE', 
			   'HMN'   => 'Maths', 
			   'HMS'   => 'Maths', 
			   'HP'    => 'HARDSCIENCE', 
			   'HPE'   => 'Physics', 
			   'HPO'   => 'Physics', 
			   'M'     => 'MEDICINE', 
			   'USA'   => 'MEDICINE', 
			   'MB'    => 'MEDICINE', 
			   'MX'    => 'MEDICINE', 
			   'MD'    => 'MEDICINE', 
			   'MDD'   => 'DrugCulture', 
			   'MH'    => 'MEDICINE', 
			   'Ml'    => 'MEDICINE', 
			   'MN'    => 'MEDICINE', 
			   'MP'    => 'MEDICINE', 
			   'P'     => 'POLITICS/WORLD AFFAIRS', 
			   'PE'    => 'POLITICS/WORLD AFFAIRS', 
			   'PEC'   => 'Economics', 
			   'PEG'   => 'Economics', 
			   'PET'   => 'Economics', 
			   'PEW'   => 'Economics', 
			   'PG'    => 'POLITICS/WORLD AFFAIRS', 
			   'PGC'   => 'Government', 
			   'POP'   => 'Government', 
			   'PGO'   => 'Government', 
			   'PM'    => 'POLITICS/WORLD AFFAIRS', 
			   'PMA'   => 'Military', 
			   'PMD'   => 'Military', 
			   'PMN'   => 'Military', 
			   'PMP'   => 'Military', 
			   'PMW'   => 'Military', 
			   'PP'    => 'POLITICS/WORLD AFFAIRS', 
			   'PPG'   => 'Politics', 
			   'PPP'   => 'Politics', 
			   'PPV'   => 'Politics', 
			   'PT'    => 'POLITICS/WORLD AFFAIRS', 
			   'R'     => 'RELIGION&THOUGHT', 
			   'RF'    => 'RELIGION&THOUGHT', 
			   'RM'    => 'RELIGION&THOUGHT', 
			   'RO'    => 'RELIGION&THOUGHT', 
			   'ROA'   => 'Occult', 
			   'ROM'   => 'Occult', 
			   'RP'    => 'RELIGION&THOUGHT', 
			   'RR'    => 'RELIGION&THOUGHT', 
			   'RRB'   => 'Religion', 
			   'RRC'   => 'Religion', 
			   'RRH'   => 'Religion', 
			   'RRI'   => 'Religion', 
			   'RRJ'   => 'Religion', 
			   'S'     => 'SOCIETY/SOFT-SCIENCE', 
			   'SA'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SAN'   => 'Anthropology', 
			   'SX'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SC'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SCB'   => 'Crime&Law', 
			   'SCC'   => 'Crime&Law', 
			   'SCJ'   => 'Crime&Law', 
			   'SCL'   => 'Crime&Law', 
			   'SCP'   => 'Crime&Law', 
			   'SCT'   => 'Crime&Law', 
			   'SE'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SEC'   => 'Education', 
			   'SEP'   => 'Education', 
			   'SES'   => 'Education', 
			   'SET'   => 'Education', 
			   'SG'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SGE'   => 'Geography', 
			   'SOP'   => 'Geography', 
			   'SH'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SL'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SLA'   => 'Linguistics', 
			   'SLG'   => 'Linguistics', 
			   'SLL'   => 'Linguistics', 
			   'SY'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SS'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SSC'   => 'Sociology', 
			   'SSF'   => 'Sociology', 
			   'SSO'   => 'Sociology', 
			   'SSR'   => 'Sociology', 
			   'SSW'   => 'Sociology', 
			   'SSY'   => 'Sociology', 
			   'T'     => 'TECHNOLOGY', 
			   'TA'    => 'TECHNOLOGY', 
			   'TAB'   => 'Agriculture', 
			   'TAC'   => 'Agriculture', 
			   'TAF'   => 'Agriculture', 
			   'TAS'   => 'Agriculture', 
			   'TB'    => 'TECHNOLOGY', 
			   'TBC'   => 'Building', 
			   'TBB'   => 'Building', 
			   'TC'    => 'TECHNOLOGY', 
			   'TCB'   => 'Communications', 
			   'TCM'   => 'Communications',  
			   'TCN'   => 'Communications', 
			   'TCP'   => 'Communications', 
			   'TCR'   => 'Communications', 
			   'TCT'   => 'Communications', 
			   'TD'    => 'Communications', 
			   'TE'    => 'TECHNOLOGY', 
			   'TEC'   => 'Engineering', 
			   'TEE'   => 'Engineering', 
			   'TEM'   => 'Engineering', 
			   'TI'    => 'TECHNOLOGY', 
			   'TIC'   => 'Industry', 
			   'TIF'   => 'Industry', 
			   'TIG'   => 'Industry', 
			   'TIM'   => 'Industry', 
			   'TIP'   => 'Industry', 
			   'TM'    => 'TECHNOLOGY', 
			   'TMC'   => 'Measurement', 
			   'TMT'   => 'Measurement', 
			   'TP'    => 'TECHNOLOGY', 
			   'TPE'   => 'Power', 
			   'TPG'   => 'Power', 
			   'TPW'   => 'Power', 
			   'TZ'    => 'TECHNOLOGY', 
			   'TT'    => 'TECHNOLOGY', 
			   'TTA'   => 'Transport', 
			   'TTB'   => 'Transport', 
			   'TTC'   => 'Transport', 
			   'TTR'   => 'Transport', 
			   'TTS'   => 'Transport', 
			   'TTT'   => 'Transport', 
			   'TTW'   => 'Transport');

my %lead_field_codes  =   ('A'     => 'ARTS', 
			   'AA'    => 'ARTS', 
			   'AL'    => 'ARTS', 
			   'AM'    => 'ARTS', 
			   'AMF'   => 'ARTS', 
			   'AMT'   => 'ARTS', 
			   'AP'    => 'ARTS', 
			   'APD'   => 'ARTS', 
			   'APM'   => 'ARTS', 
			   'APT'   => 'ARTS', 
			   'AV'    => 'ARTS', 
			   'AVD'   => 'ARTS', 
			   'AVP'   => 'ARTS', 
			   'AVS'   => 'ARTS', 
			   'B'     => 'BUSINESS', 
			   'BB'    => 'BUSINESS', 
			   'BBA'   => 'BUSINESS', 
			   'BBC'   => 'BUSINESS', 
			   'BBB'   => 'BUSINESS', 
			   'REM'   => 'BUSINESS', 
			   'BBO'   => 'BUSINESS', 
			   'RET'   => 'BUSINESS', 
			   'BE'    => 'BUSINESS', 
			   'BEC'   => 'BUSINESS', 
			   'BEL'   => 'BUSINESS', 
			   'BEW'   => 'BUSINESS', 
			   'BF'    => 'BUSINESS', 
			   'BFB'   => 'BUSINESS', 
			   'BFL'   => 'BUSINESS', 
			   'BFS'   => 'BUSINESS', 
			   'BFI'   => 'BUSINESS', 
			   'BO'    => 'BUSINESS', 
			   'C'     => 'COLOURS/SOUNDS/ETC', 
			   'CC'    => 'COLOURS/SOUNDS/ETC', 
			   'CF'    => 'COLOURS/SOUNDS/ETC', 
			   'CO'    => 'COLOURS/SOUNDS/ETC', 
			   'CS'    => 'COLOURS/SOUNDS/ETC', 
			   'CT'    => 'COLOURS/SOUNDS/ETC', 
			   'D'     => 'DAILYLIFE', 
			   'DC'    => 'DAILYLIFE', 
			   'DCB'   => 'DAILYLIFE', 
			   'DCC'   => 'DAILYLIFE', 
			   'DCJ'   => 'DAILYLIFE', 
			   'DF'    => 'DAILYLIFE', 
			   'DFC'   => 'DAILYLIFE', 
			   'DFD'   => 'DAILYLIFE', 
			   'DFF'   => 'DAILYLIFE', 
			   'DFN'   => 'DAILYLIFE', 
			   'DFT'   => 'DAILYLIFE', 
			   'DFU'   => 'DAILYLIFE', 
			   'DG'    => 'DAILYLIFE', 
			   'DOB'   => 'DAILYLIFE', 
			   'DGC'   => 'DAILYLIFE', 
			   'DOD'   => 'DAILYLIFE', 
			   'DOG'   => 'DAILYLIFE', 
			   'DGO'   => 'DAILYLIFE', 
			   'DOS'   => 'DAILYLIFE', 
			   'DH'    => 'DAILYLIFE', 
			   'DHB'   => 'DAILYLIFE', 
			   'DHC'   => 'DAILYLIFE', 
			   'DHF'   => 'DAILYLIFE', 
			   'DHH'   => 'DAILYLIFE', 
			   'DHP'   => 'DAILYLIFE', 
			   'DHT'   => 'DAILYLIFE', 
			   'DL'    => 'DAILYLIFE', 
			   'DLA'   => 'DAILYLIFE', 
			   'DLC'   => 'DAILYLIFE', 
			   'DLG'   => 'DAILYLIFE', 
			   'DLH'   => 'DAILYLIFE', 
			   'DLO'   => 'DAILYLIFE', 
			   'DLT'   => 'DAILYLIFE', 
			   'DN'    => 'DAILYLIFE', 
			   'DS'    => 'DAILYLIFE', 
			   'DSA'   => 'DAILYLIFE', 
			   'DSB'   => 'DAILYLIFE', 
			   'DSC'   => 'DAILYLIFE', 
			   'DSF'   => 'DAILYLIFE', 
			   'DSG'   => 'DAILYLIFE', 
			   'DSH'   => 'DAILYLIFE', 
			   'DSO'   => 'DAILYLIFE', 
			   'DSS'   => 'DAILYLIFE', 
			   'DST'   => 'DAILYLIFE', 
			   'H'     => 'HARDSCIENCE', 
			   'HA'    => 'HARDSCIENCE', 
			   'HB'    => 'HARDSCIENCE', 
			   'HBA'   => 'HARDSCIENCE', 
			   'HBB'   => 'HARDSCIENCE', 
			   'HBF'   => 'HARDSCIENCE', 
			   'HBH'   => 'HARDSCIENCE', 
			   'HBI'   => 'HARDSCIENCE', 
			   'HBM'   => 'HARDSCIENCE', 
			   'HBP'   => 'HARDSCIENCE', 
			   'HC'    => 'HARDSCIENCE', 
			   'HCC'   => 'HARDSCIENCE', 
			   'HCE'   => 'HARDSCIENCE', 
			   'HCM'   => 'HARDSCIENCE', 
			   'HCP'   => 'HARDSCIENCE', 
			   'HE'    => 'HARDSCIENCE', 
			   'MEG'   => 'HARDSCIENCE', 
			   'HEG'   => 'HARDSCIENCE', 
			   'HEO'   => 'HARDSCIENCE', 
			   'HM'    => 'HARDSCIENCE', 
			   'HMN'   => 'HARDSCIENCE', 
			   'HMS'   => 'HARDSCIENCE', 
			   'HP'    => 'HARDSCIENCE', 
			   'HPE'   => 'HARDSCIENCE', 
			   'HPO'   => 'HARDSCIENCE', 
			   'M'     => 'MEDICINE', 
			   'USA'   => 'MEDICINE', 
			   'MB'    => 'MEDICINE', 
			   'MX'    => 'MEDICINE', 
			   'MD'    => 'MEDICINE', 
			   'MDD'   => 'MEDICINE', 
			   'MH'    => 'MEDICINE', 
			   'Ml'    => 'MEDICINE', 
			   'MN'    => 'MEDICINE', 
			   'MP'    => 'MEDICINE', 
			   'P'     => 'POLITICS/WORLD AFFAIRS', 
			   'PE'    => 'POLITICS/WORLD AFFAIRS', 
			   'PEC'   => 'POLITICS/WORLD AFFAIRS', 
			   'PEG'   => 'POLITICS/WORLD AFFAIRS', 
			   'PET'   => 'POLITICS/WORLD AFFAIRS', 
			   'PEW'   => 'POLITICS/WORLD AFFAIRS', 
			   'PG'    => 'POLITICS/WORLD AFFAIRS', 
			   'PGC'   => 'POLITICS/WORLD AFFAIRS', 
			   'POP'   => 'POLITICS/WORLD AFFAIRS', 
			   'PGO'   => 'POLITICS/WORLD AFFAIRS', 
			   'PM'    => 'POLITICS/WORLD AFFAIRS', 
			   'PMA'   => 'POLITICS/WORLD AFFAIRS', 
			   'PMD'   => 'POLITICS/WORLD AFFAIRS', 
			   'PMN'   => 'POLITICS/WORLD AFFAIRS', 
			   'PMP'   => 'POLITICS/WORLD AFFAIRS', 
			   'PMW'   => 'POLITICS/WORLD AFFAIRS', 
			   'PP'    => 'POLITICS/WORLD AFFAIRS', 
			   'PPG'   => 'POLITICS/WORLD AFFAIRS', 
			   'PPP'   => 'POLITICS/WORLD AFFAIRS', 
			   'PPV'   => 'POLITICS/WORLD AFFAIRS', 
			   'PT'    => 'POLITICS/WORLD AFFAIRS', 
			   'R'     => 'RELIGION&THOUGHT', 
			   'RF'    => 'RELIGION&THOUGHT', 
			   'RM'    => 'RELIGION&THOUGHT', 
			   'RO'    => 'RELIGION&THOUGHT', 
			   'ROA'   => 'RELIGION&THOUGHT', 
			   'ROM'   => 'RELIGION&THOUGHT', 
			   'RP'    => 'RELIGION&THOUGHT', 
			   'RR'    => 'RELIGION&THOUGHT', 
			   'RRB'   => 'RELIGION&THOUGHT', 
			   'RRC'   => 'RELIGION&THOUGHT', 
			   'RRH'   => 'RELIGION&THOUGHT', 
			   'RRI'   => 'RELIGION&THOUGHT', 
			   'RRJ'   => 'RELIGION&THOUGHT', 
			   'S'     => 'SOCIETY/SOFT-SCIENCE', 
			   'SA'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SAN'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SX'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SC'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SCB'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SCC'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SCJ'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SCL'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SCP'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SCT'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SE'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SEC'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SEP'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SES'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SET'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SG'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SGE'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SOP'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SH'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SL'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SLA'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SLG'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SLL'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SY'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SS'    => 'SOCIETY/SOFT-SCIENCE', 
			   'SSC'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SSF'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SSO'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SSR'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SSW'   => 'SOCIETY/SOFT-SCIENCE', 
			   'SSY'   => 'SOCIETY/SOFT-SCIENCE', 
			   'T'     => 'TECHNOLOGY', 
			   'TA'    => 'TECHNOLOGY', 
			   'TAB'   => 'TECHNOLOGY', 
			   'TAC'   => 'TECHNOLOGY', 
			   'TAF'   => 'TECHNOLOGY', 
			   'TAS'   => 'TECHNOLOGY', 
			   'TB'    => 'TECHNOLOGY', 
			   'TBC'   => 'TECHNOLOGY', 
			   'TBB'   => 'TECHNOLOGY', 
			   'TC'    => 'TECHNOLOGY', 
			   'TCB'   => 'TECHNOLOGY', 
			   'TCM'   => 'TECHNOLOGY',  
			   'TCN'   => 'TECHNOLOGY', 
			   'TCP'   => 'TECHNOLOGY', 
			   'TCR'   => 'TECHNOLOGY', 
			   'TCT'   => 'TECHNOLOGY', 
			   'TD'    => 'TECHNOLOGY', 
			   'TE'    => 'TECHNOLOGY', 
			   'TEC'   => 'TECHNOLOGY', 
			   'TEE'   => 'TECHNOLOGY', 
			   'TEM'   => 'TECHNOLOGY', 
			   'TI'    => 'TECHNOLOGY', 
			   'TIC'   => 'TECHNOLOGY', 
			   'TIF'   => 'TECHNOLOGY', 
			   'TIG'   => 'TECHNOLOGY', 
			   'TIM'   => 'TECHNOLOGY', 
			   'TIP'   => 'TECHNOLOGY', 
			   'TM'    => 'TECHNOLOGY', 
			   'TMC'   => 'TECHNOLOGY', 
			   'TMT'   => 'TECHNOLOGY', 
			   'TP'    => 'TECHNOLOGY', 
			   'TPE'   => 'TECHNOLOGY', 
			   'TPG'   => 'TECHNOLOGY', 
			   'TPW'   => 'TECHNOLOGY', 
			   'TZ'    => 'TECHNOLOGY', 
			   'TT'    => 'TECHNOLOGY', 
			   'TTA'   => 'TECHNOLOGY', 
			   'TTB'   => 'TECHNOLOGY', 
			   'TTC'   => 'TECHNOLOGY', 
			   'TTR'   => 'TECHNOLOGY', 
			   'TTS'   => 'TECHNOLOGY', 
			   'TTT'   => 'TECHNOLOGY', 
			   'TTW'   => 'TECHNOLOGY');

my %field_codes = ('A'     => 'ARTS', 
		   'AA'    => 'Architecture', 
		   'AL'    => 'Literary', 
		   'AM'    => 'Media', 
		   'AMF'   => 'Film', 
		   'AMT'   => 'Television&Radio', 
		   'AP'    => 'Performing', 
		   'APD'   => 'Dance', 
		   'APM'   => 'Music', 
		   'APT'   => 'Theatre', 
		   'AV'    => 'Visual', 
		   'AVD'   => 'Design', 
		   'AVP'   => 'Painting', 
		   'AVS'   => 'Sculpture', 
		   'B'     => 'BUSINESS', 
		   'BB'    => 'Basics', 
		   'BBA'   => 'Advertising', 
		   'BBC'   => 'Companies', 
		   'BBB'   => 'BusinessManagement', 
		   'REM'   => 'Marketing', 
		   'BBO'   => 'Offices', 
		   'RET'   => 'Trade', 
		   'BE'    => 'Employment', 
		   'BEC'   => 'ConditionsofEmployment', 
		   'BEL'   => 'LabourRelations/Unions', 
		   'BEW'   => 'Wages', 
		   'BF'    => 'Finance', 
		   'BFB'   => 'Banking', 
		   'BFL'   => 'Loans', 
		   'BFS'   => 'Stocks&Shares', 
		   'BFI'   => 'Insurance', 
		   'BO'    => 'Occupations', 
		   'C'     => 'COLOURS/SOUNDS/ETC', 
		   'CC'    => 'Colours', 
		   'CF'    => 'Forms/shapes', 
		   'CO'    => 'Odours', 
		   'CS'    => 'Surface/Texture', 
		   'CT'    => 'Tastes', 
		   'D'     => 'DAILYLIFE', 
		   'DC'    => 'Clothes&Fashion', 
		   'DCB'   => 'Beauty', 
		   'DCC'   => 'Clothes', 
		   'DCJ'   => 'Jewellery', 
		   'DF'    => 'Food', 
		   'DFC'   => 'Cooking', 
		   'DFD'   => 'Drink', 
		   'DFF'   => 'FoodDish', 
		   'DFN'   => 'Nutrition', 
		   'DFT'   => 'Tobacco', 
		   'DFU'   => 'Utensils', 
		   'DG'    => 'Games', 
		   'DOB'   => 'BoardGames', 
		   'DGC'   => 'Cards', 
		   'DOD'   => 'Darts', 
		   'DOG'   => 'Gambling', 
		   'DGO'   => 'OtherGames', 
		   'DOS'   => 'Snooker', 
		   'DH'    => 'Household', 
		   'DHB'   => 'Babies', 
		   'DHC'   => 'Cleaning', 
		   'DHF'   => 'Furniture', 
		   'DHH'   => 'House', 
		   'DHP'   => 'Pets', 
		   'DHT'   => 'Toys', 
		   'DL'    => 'Leisure', 
		   'DLA'   => 'Antiques', 
		   'DLC'   => 'Clubs', 
		   'DLG'   => 'Gardening', 
		   'DLH'   => 'Hobbies', 
		   'DLO'   => 'Outdoor', 
		   'DLT'   => 'Tourism', 
		   'DN'    => 'Nature', 
		   'DS'    => 'Sport', 
		   'DSA'   => 'AmericanFootball', 
		   'DSB'   => 'Baseball', 
		   'DSC'   => 'Cricket', 
		   'DSF'   => 'Football', 
		   'DSG'   => 'Golf', 
		   'DSH'   => 'Horses', 
		   'DSo'   => 'Other Sports', 
		   'DSS'   => 'Swimming', 
		   'DST'   => 'Tennis', 
		   'H'     => 'HARDSCIENCE', 
		   'HA'    =>  'Astronomy', 
		   'HB'    => 'Biology', 
		   'HBA'   => 'Animals', 
		   'HBB'   => 'Birds', 
		   'HBF'   => 'Fish', 
		   'HBH'   => 'Human', 
		   'HBI'   => 'Insects', 
		   'HBM'   => 'Microbes/Genetics/Biochem', 
		   'HBP'   => 'Plants', 
		   'HC'    => 'Chemistry', 
		   'HCC'   => 'Compounds', 
		   'HCE'   => 'Elements', 
		   'HCM'   => 'Metals', 
		   'HCP'   => 'Plastics', 
		   'HE'    => 'EarthSciences', 
		   'MEG'   => 'Geology', 
		   'HEG'   => 'Meteorology', 
		   'HEO'   => 'Oceanography', 
		   'HM'    => 'Maths', 
		   'HMN'   => 'Numbers', 
		   'HMS'   => 'Statistics', 
		   'HP'    => 'Physics', 
		   'HPE'   => 'Electricity', 
		   'HPO'   => 'Optics', 
		   'M'     => 'MEDICINE', 
		   'USA'   => 'AlternativeMedicine', 
		   'MB'    => 'Birth', 
		   'MX'    => 'Death', 
		   'MD'    => 'Drugs/Medicines', 
		   'MDD'   => 'DrugCulture', 
		   'MH'    => 'Hospital', 
		   'Ml'    => 'Illness&Disability', 
		   'MN'    => 'Nurses,Doctors,etc', 
		   'MP'    => 'Psychology&Psychiatry', 
		   'P'     => 'POLITICS/WORLD AFFAIRS', 
		   'PE'    => 'Economics', 
		   'PEC'   => 'Currencies', 
		   'PEG'   => 'Grants', 
		   'PET'   => 'Tax', 
		   'PEW'   => 'Welfare', 
		   'PG'    => 'Government', 
		   'PGC'   => 'Citizenship', 
		   'POP'   => 'Parliaments', 
		   'PGO'   => 'Officials', 
		   'PM'    => 'Military', 
		   'PMA'   => 'Army', 
		   'PMD'   => 'Defence', 
		   'PMN'   => 'Navy', 
		   'PMP'   => 'Planes(airforce)', 
		   'PMW'   => 'Weapons', 
		   'PP'    => 'Politics', 
		   'PPG'   => 'Groupings', 
		   'PPP'   => 'Policies', 
		   'PPV'   => 'Voting', 
		   'PT'    => 'ThirdWorld', 
		   'R'     => 'RELIGION&THOUGHT', 
		   'RF'    => 'Folklore', 
		   'RM'    => 'Mythology', 
		   'RO'    => 'Occult', 
		   'ROA'   => 'Astrology', 
		   'ROM'   => 'Magic', 
		   'RP'    => 'Philosophy', 
		   'RR'    => 'Religion', 
		   'RRB'   => 'Buddhism', 
		   'RRC'   => 'Christianity', 
		   'RRH'   => 'Hinduism', 
		   'RRI'   => 'Islam', 
		   'RRJ'   => 'Judaism', 
		   'S'     => 'SOCIETY/SOFT-SCIENCE', 
		   'SA'    => 'Anthropology', 
		   'SAN'   => 'Nationalities', 
		   'SX'    => 'Archaeology', 
		   'SC'    => 'Crime&Law', 
		   'SCB'   => 'Bombs&Terrorism', 
		   'SCC'   => 'Crime', 
		   'SCJ'   => 'Jail', 
		   'SCL'   => 'Law', 
		   'SCP'   => 'Police', 
		   'SCT'   => 'Trials', 
		   'SE'    => 'Education', 
		   'SEC'   => 'College', 
		   'SEP'   => 'PreSchool', 
		   'SES'   => 'School', 
		   'SET'   => 'Training', 
		   'SG'    => 'Geography', 
		   'SGE'   => 'Ecology', 
		   'SOP'   => 'Pollution', 
		   'SH'    => 'History', 
		   'SL'    => 'Linguistics', 
		   'SLA'   => 'Alphabets', 
		   'SLG'   => 'Grammar', 
		   'SLL'   => 'Languages', 
		   'SY'    => 'Sex', 
		   'SS'    => 'Sociology', 
		   'SSC'   => 'Children', 
		   'SSF'   => 'Family', 
		   'SSO'   => 'Organisations', 
		   'SSR'   => 'RaceRelations', 
		   'SSW'   => 'Women', 
		   'SSY'   => 'Youth', 
		   'T'     => 'TECHNOLOGY', 
		   'TA'    => 'Agriculture', 
		   'TAB'   => 'Biotechnology', 
		   'TAC'   => 'Crops', 
		   'TAF'   => 'Forestry', 
		   'TAS'   => 'Soil', 
		   'TB'    => 'Building:', 
		   'TBC'   => 'Construction', 
		   'TBB'   => 'Buildings', 
		   'TC'    => 'Communications', 
		   'TCB'   => 'Broadcasting', 
		   'TCM'   => 'Mail',  
		   'TCN'   => 'Newspapers/Printing/Publishing', 
		   'TCP'   => 'Photography', 
		   'TCR'   => 'Recording', 
		   'TCT'   => 'TelephoneTelegraph', 
		   'TD'    => 'DataProcessing&Computing', 
		   'TE'    => 'Engineering', 
		   'TEC'   => 'Civil', 
		   'TEE'   => 'Electrical', 
		   'TEM'   => 'Mechanical', 
		   'TI'    => 'Industry', 
		   'TIC'   => 'Crafts', 
		   'TIF'   => 'Factories', 
		   'TIG'   => 'Glass', 
		   'TIM'   => 'Material&Textiles', 
		   'TIP'   => 'Pottery&Ceramics', 
		   'TM'    => 'Measurement', 
		   'TMC'   => 'Chronology', 
		   'TMT'   => 'Temperature', 
		   'TP'    => 'Power', 
		   'TPE'   => 'Electricity', 
		   'TPG'   => 'GasCoalOil', 
		   'TPW'   => 'WindWaterSun', 
		   'TZ'    => 'Tools', 
		   'TT'    => 'Transport', 
		   'TTA'   => 'Air', 
		   'TTB'   => 'Bicycles&Cans', 
		   'TTC'   => 'Cars', 
		   'TTR'   => 'Roads', 
		   'TTS'   => 'Space', 
		   'TTT'   => 'Trains&Railways', 
		   'TTW'   => 'Water'); 

my %grammatical_codes = ('A'   => 'Agent',
			 'T'   => 'Theme',
			 'CA'  => 'Causal Agent',
			 'I'   => 'Instrument',
			 'ATB' => 'Attributive',
			 'E'   => 'Experiencer',
			 'B'   => 'Benenficiary');

#returns words with the same subject codes as the query word
sub subjectcodes
{
    my $self = shift;
    my $word = shift;
    my @chars = split(//, $word);
    my $file = "";

    if(lc($chars[0]) eq 'e') { $file = $location . lc($chars[0]) . ".sgm"; } 
    else { $file = $location . uc($chars[0]) . ".SGM"; }                     
    
    open(FILE, $file) || die;                                                
    
    my %temp = (); 
    my $w = "";
    while(<FILE>) {
 	if($_=~m/<Entry><Head><HWD>([A-Za-z]+)<\/HWD>/) { $w = $1; }
	if( ($_=~m/<FIELD>([A-Z]+)<\/FIELD>/) && ($w eq $word) ) { $temp{$1}++; } 
    }
    
    close(FILE);
    
    my %hash = ();
    foreach my $t (sort keys %temp ) { 
	open(INDEX, "index") || die;
	while(<INDEX>) {
	    my @code = split/\s+/, $_; 
	    if($t eq $code[1]) {
		my $index = $word . " " . $t;
		push @{ $hash{$index} }, $code[0];
	    }
	}
	close(INDEX);
    }

    return %hash;
    
}

#return words with the same active code as the query word
sub activecodes
{
    my $self = shift;
    my $word = shift;
    my @chars = split(//, $word);
    my $file = "";

    if(lc($chars[0]) eq 'e') { $file = $location . lc($chars[0]) . ".sgm"; } 
    else { $file = $location . uc($chars[0]) . ".SGM"; }                     
    
    open(FILE, $file) || die;                                                
    
    my %temp = (); 
    my $w = "";
    while(<FILE>) {
 	if($_=~m/<Entry><Head><HWD>([A-Za-z]+)<\/HWD>/) { $w = $1; }
	if( ($_=~m/<ACTIV>([A-Z]+)<\/ACTIV>/) && ($w eq $word) ) { $temp{$1}++; } 
    }
    
    close(FILE);
    
    my %hash = ();
    foreach my $t (sort keys %temp ) { 
	open(ACTIVE, "active") || die;
	while(<ACTIVE>) {
	    my @code = split/\s+/, $_; 
	    if($t eq $code[1]) { 
		my $index = $word . " " . $t;
		push @{ $hash{$index} }, $code[0];
	    }
	}
	close(ACTIVE);
    }

    return %hash;
    
}

my %pos;
#return words with the same part of speech as the query word
sub partofspeech
{
    my $self = shift;
    my $word = shift;
    my @chars = split(//, $word);
    my $file = "";
    
    if(lc($chars[0]) eq 'e') { $file = $location . lc($chars[0]) . ".sgm"; }
    else { $file = $location . uc($chars[0]) . ".SGM"; }
    
    open(FILE, $file) || die;
    
    my %temp = ();
    my $w = "";
    while(<FILE>) {
 	if($_=~m/<Entry><Head><HWD>(.+)<\/HWD>/) { $w = $1; }
	if($w eq $word) { if($_=~m/<POS>(.+)<\/POS>/) { $temp{$1}++; } }
    }
    close(FILE);
   
    unless(%pos) { 
	open(POS, "pos");
	while(<POS>) {
	    chomp;
	    my @code = split /\:/, $_;
	    push @{ $pos{$code[1]} }, $code[0];
	}
	foreach my $p (sort keys %pos) {
	    my %check = ();
	    for my $i (0..$#{ $pos{$p} }) {
		$check{$pos{$p}[$i]}++;
	    }
	    my @checker = sort keys %check;
	    $pos{$p} = [ @checker ];
	}
    }
    
   my %hash = ();
    
    foreach my $t (sort keys %temp) { 
	my $index = $word . " " . $t;
	$hash{$index} = [ @{ $pos{$t} } ];
    }

    return %hash;
}

#returns if word is an adjective or not
sub isadjective
{
    my $self = shift;
    my $word = shift;
    
    my $abriev = "adj";

    unless(%pos) { 
	open(POS, "pos");
	while(<POS>) {
	    chomp;
	    my @code = split /\:/, $_;
	    push @{ $pos{$code[1]} }, $code[0];
	}
	foreach my $p (sort keys %pos) {
	    my %check = ();
	    for my $i (0..$#{ $pos{$p} }) {
		$check{$pos{$p}[$i]}++;
	    }
	    my @checker = sort keys %check;
	    $pos{$p} = [ @checker ];
	}
    }
     
    my $found = 0;
    for my $i (0..$#{ $pos{$abriev} } ) {
	if($word eq $pos{$abriev}[$i]) { $found = 1; last; }
    }
   
    return $found;
}

#return definitions of words that contain the query word
sub returndefinitions
{  
    my $self = shift;
    my $word = shift;

    my @words = split/\,/, $word;

    my @sgmfiles = ( 'A.SGM','B.SGM','C.SGM','D.SGM','e.sgm','F.SGM','G.SGM',
		     'H.SGM','I.SGM','J.SGM','K.SGM','L.SGM','M.SGM','N.SGM',
		     'O.SGM','P.SGM','Q.SGM','R.SGM','S.SGM','T.SGM','U.SGM',
		     'V.SGM','W.SGM','X.SGM','Y.SGM','Z.SGM');
   
    my %array = ();

    foreach my $sgmfile (@sgmfiles) {
	$sgmfile = $location . $sgmfile;
	open(SGM, $sgmfile) || die(print "$sgmfile \n");
	while(<SGM>) {
	    if($_=~m/<Entry><Head><HWD>([A-Za-z]+)<\/HWD>/) {                     
		my($w, $t, $d, $p, $f);
		$w = $1;                                                          
		$t = <SGM>;                                                       
		while($t !~ /<\/Entry>/) {                                        
		     if($t=~m/<POS>(.+)<\/POS>/) { $p = $1; }            
		     if($t=~m/<FIELD>([A-Za-z]+)<\/FIELD>/) { $f = $1; }         
		     if($t=~m/<DEF>(.+)<\/DEF>/) {                               
			 $d = $1;
			 my $found = 1;
			 foreach my $worten (@words) {
			     if($d=~/\b$worten\b/) { } else { $found = 0; }
			 }
			 if($found == 1) {
			     my $temp = $w . ", " . $p . ", " . $f . " : " . $d;
			     $array{$temp}++;                                    
			 }
		     }
		     $t = <SGM>;                                                  
		 }
	    }
	}
    }
    return %array;
}

# query for the set word and return the code information
sub query
{
    #initialize variables
    my $self = shift;
    my $word = shift;
    my $code = shift;
    my @chars = split(//, $word);
    my $file = "";
   
    #set the files
    if(lc($chars[0]) eq 'e') { $file = $location . lc($chars[0]) . ".sgm"; }
    else { $file = $location . uc($chars[0]) . ".SGM"; }
    
    #initialize variables
    my $t = "";
    my @temp = ();
    
    #get pos if exists
    my @splitting = split/\#/, $word;
    $word = $splitting[0];
    my $partofspeech = $splitting[1];

    #set the code
    my $check = "";
    if($code eq "ex")  { $check = "EXAMPLE"; }
    elsif($code eq "def") { $check = "DEF"; }
    elsif($code eq "spell") { $check = "SPELLING"; }
    elsif($code eq "lexunit") { $check = "LEXUNIT"; }

    #query
    open(FILE, $file);
    while(<FILE>) {
	if( ($_=~m/<Entry><Head><HWD>$word<\/HWD>/) || ($t=~m/<Entry><Head><HWD>$word<\/HWD>/) ) {                    
	    @temp = "";                                               
	    $t = <FILE>;                                              
	    if($t=~ /<$check>(.+)<\/$check>/) { push @temp, $t; }      
	    if($t=~/<POS>(.+)<\/POS>/) { push @temp, $t; }         
	    while($t !~ /<\/Entry>/) {                                 
		$t = <FILE>;                                           
		if($t=~ /<$check>(.+)<\/$check>/) { push @temp, $t; }  
		if($t=~/<POS>(.+)<\/POS>/) { push @temp, $t; }    
	    }
	}
    }
    close(FILE);
    print @temp;
    #initialize variables
    my ($p, $d);
    my %hash = ();
    #set hash
    foreach my $line (@temp) {                           
	if($line=~/<POS>(.+)<\/POS>/) { $p = $1; } 
	if($line=~/<$check>(.+)<\/$check>/) {         
	    $d = $1;                               
	    $d=~s/<.+>//gi;                           
	    my $t = $p . ":" . $d;                   
	    $hash{$t}++;                            
	}
	
    }
    #return hash depending upon whether pos was included in query
    my %returnhash;
    foreach my $index (sort keys %hash) {
	my @array = split/:/, $index;
	if($partofspeech eq "") { push @{ $returnhash{$array[0]} }, $array[1]; }
	if($partofspeech eq $array[0]) {  push @{ $returnhash{$array[0]} }, $array[1]; }
    }
    
    return %returnhash;
} 
#get the active code for a given word
sub getactivecode
{
    #initialize variables
    my $self = shift;
    my $word = shift;
    my $code = shift;
    my @chars = split(//, $word);
    my $file = "";
    my @temp = ();
    
    #set file
    if(lc($chars[0]) eq 'e') { $file = $location . lc($chars[0]) . ".sgm"; }
    else { $file = $location . uc($chars[0]) . ".SGM"; }
    
    #query
    open(FILE, $file);
    while(<FILE>) {
 	if($_=~m/<Entry><Head><HWD>$word<\/HWD>/) {                    
	    @temp = "";                                                
	    my $t = <FILE>;                                           
	    if($t=~m/<ACTIV>([A-Za-z]+)<\/ACTIV>/) { push @temp, $1; }    
	    while($t !~ /<\/Entry>/) {                                
		$t = <FILE>;                                           
		if($t=~m/<ACTIV>([A-Z]+)<\/ACTIV>/) { push @temp, $1; } 
	    }
	}
    }
    
    #set hash depending upon code
    my %check = ();
    foreach my $t (@temp) { if($t ne "") { $check{$t}++; } }
    return %check;
}
#get the first degree, parent or lead subject code for a given word
sub getsubjectcode
{
    #initialize variables
    my $self = shift;
    my $word = shift;
    my $code = shift;
    my @chars = split(//, $word);
    my $file = "";
    my @temp = ();
    
    #set file
    if(lc($chars[0]) eq 'e') { $file = $location . lc($chars[0]) . ".sgm"; }
    else { $file = $location . uc($chars[0]) . ".SGM"; }
    
    #query
    open(FILE, $file);
    while(<FILE>) {
 	if($_=~m/<Entry><Head><HWD>$word<\/HWD>/) {                    
	    @temp = "";                                                
	    my $t = <FILE>;                                           
	    if($t=~m/<FIELD>([A-Z]+)<\/FIELD>/) { push @temp, $1; }    
	    while($t !~ /<\/Entry>/) {                                
		$t = <FILE>;                                           
		if($t=~m/<FIELD>([A-Z]+)<\/FIELD>/) { push @temp, $1; } 
	    }
	}
    }
    
    #set hash depending upon code
    my %check = ();
    foreach my $t (@temp) {
	if($code eq "f") { if($t ne "") { $check{$field_codes{$t}}++;        } }
	if($code eq "p") { if($t ne "") { $check{$parent_field_codes{$t}}++; } }
	if($code eq "l") { if($t ne "") { $check{$lead_field_codes{$t}}++;   } }
    }
    return %check;
}

#return collocations of a given word
sub getcollocations
{
    my $self = shift;
    my $word = shift;
    my @chars = split(//, $word);
    my $file = "";
    my %temp = ();

    if(lc($chars[0]) eq 'e') { $file = $location . lc($chars[0]) . ".sgm"; }
    else { $file = $location . uc($chars[0]) . ".SGM"; }
    
    open(FILE, $file);

    while(<FILE>) {
 	if($_=~m/<Entry><Head><HWD>$word<\/HWD>/) {             
	    my $t = <FILE>;                                     
	    if($t=~m/<COLLO>(.+)<\/COLLO>/) { $temp{$1}++; }     
	    while($t !~ /<\/Entry>/) {                           
		$t = <FILE>;                                    
		if($t=~m/<COLLO>(.+)<\/COLLO>/) { $temp{$1}++; } 
	    }
	}
    }
    
    return %temp;
}

sub spelling
{
    my $self = shift;
    my $word = shift;
    
    my @chars = split(//, $word);
    my $file = "";
    my %temp = ();

    if(lc($chars[0]) eq 'e') { $file = $location . lc($chars[0]) . ".sgm"; }
    else { $file = $location . uc($chars[0]) . ".SGM"; }
    
    open(FILE, $file);

    while(<FILE>) {
 	if($_=~m/<Entry><Head><HWD>$word<\/HWD>/) {             
	    my $t = <FILE>;                                      
	    if($t=~m/<COLLO>(.+)<\/COLLO>/) { $temp{$1}++; }    
	    while($t !~ /<\/Entry>/) {                          
		$t = <FILE>;                                     
		if($t=~m/<COLLO>(.+)<\/COLLO>/) { $temp{$1}++; } 
	    }
	}
    }
    
    return %temp;
}
1;    

__END__

#################
# Documentation #
#################

=head1 NAME

LDOCE - direct perl interface to Longman's machine readable dictionary (LDOCE)

=head1 SYNOPSIS

  use Minas::LDOCE;

  $word = "word";

  my $l = Minas::LDOCE->new("/home/bthomson/LDOCE3/A-Z/sgml/");
  
  %temp = $ldoce->activecodes($word);
  %temp = $ldoce->subjectcodes($word);
  %temp = $ldoce->partofspeech($word);
  %temp = $ldoce->returndefinitions($word);
  %temp = $ldoce->query($word, "def"); 
  %temp = $ldoce->query($word, "ex");
  %temp = $ldoce->query($word, "spell");
  %temp = $ldoce->query($word, "lexunit"); 
  %temp = $ldoce->getsubjectcode($word, $code);
  %temp = $ldoce->getcollocations($word);
  %temp = $ldoce->getactivecode($word);
  $temp = $ldoce->isadjective($word);

=head1 DESCRIPTION

LDOCE provides a direct interface to the LDOCE3 SGM files.
  
It requires the LDOCE3 package. LDOCE3 is a machine readable dictionary.

It allows the user direct access to the full LDOCE SGM file.  

=head1 USAGE

    use LDOCE;

=head2 LOCATING THE LDOCE3 MACHINE READABLE DICTIONARY

To use ldoce, you must tell it where your LDOCE3 SGM files are.
You can do this by passing the location to ldoce when you invoke 
the "new" function.

To do this, pass the LDOCE3s location to "new":

  my $l = LDOCE->new("/usr/local/LDOCE3/A-Z/sgml/")

=head2 QUERYING THE MACHINE READABLE DICTIONARY, LDOCE3

There are a few different functions that can be called to query
the LDOCE3 database.

1. activecodes
   
    The function 'activecodes' returns a hash of arrays
    containing all of the words in the dictionary that 
    have the same active code as the query word. 

2. subjectcodes

    The function 'subjectcodes' returns a hash of arrays 
    containing all of the words in the dictionary that 
    have the same subject code as the query word. 

3. partofspeech

    The function 'partofspeech' returns a hash of arrays
    containing all of the words in the dictionary that have 
    the same part of speech as the query word.

4. returndefinitions
    
    The function 'returndefinitions' returns a hash containing
    all of the definitions in the dictionary that contain the
    query word.  The definition returned in the hash will contain:
       
       '<word>, <pos>, <subject code> : <definition of word>'

    as its key. Please note that word is not the query word.

5. query

    The function 'query' must be given a word and a code. The possible
    codes are:
    
        'def'     : returns the definitions of the word + pos
        'ex'      : returns examples of how the word is used + pos
        'spell'   : returns different spellings of the word + pos
        'lexunit' : return different lexical information about the 
                    word + pos

    The function returns a hash of arrays containing the information 
    specified by the code.

6. getsubjectcode

    The function 'getsubjectcode' must be given a word and a code.
    The possible codes are:

        'f' : first degree subject code
        'p' : parent subject code
        'l' : lead subject code

    More information on the subject codes are in the LDOCE.DOC manual
    located with in the LDOCE3 directory.

    The function returns a hash containing the specified subject codes
    for the query word.

7. getcollocations

    The function 'getcollocations' returns a hash of all the collocations
    that contain the query word.

8. getactivecode

    The function 'getactivecode' returns a hash of all the active codes 
    for the query word.

9  isadjective

    The function 'isadjective' returns a boolean true (1) or false (0)
    depending upon whether the word given is an adjective.

10. new
    
    new creates an instance of the LDOCE.pm. There are four
    things that happen in new.
    
      1. the location of the machine readable dictionary is set
      2. the pos file, 'pos', is created
      3. the index file, 'index', is created
      4. the active file, 'active', is created

    Please note that these files are necessary for the running of
    LDOCE.pm. They will be created if they do not exist.


An example of how to access a hash of arrays is:

	foreach $d (sort keys %def) {
            for $i (0..$#{ $def{$d} } ) {
                print "   $d => $def{$d}[$i] \n";
            }
        }

Other examples are located in the program 'interface.pl'.

    
=head1 NOTES

Requires access to LDOCE3 SGM files

=head1 COPYRIGHT

Copyright 2002 Bridget Thomson McInnes, Yanhua Li, Deodatta Bhoite, Kailash 
Aurangabadkar, Nitin Agarwal.  All rights reserved.

This module is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 SEE ALSO

perl(1)
